@@ -1,6 +1,6 @@
 /*
 htop - Process.c
-(C) 2004-2006 Hisham H. Muhammad
+(C) 2004-2010 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
@@ -27,19 +27,33 @@ in the source distribution for its full text.
 #include <stdbool.h>
 #include <pwd.h>
 #include <sched.h>
+#include <time.h>
 
+#ifdef HAVE_PLPA
 #include <plpa.h>
+#endif
 
 // This works only with glibc 2.1+. On earlier versions
 // the behavior is similar to have a hardcoded page size.
 #ifndef PAGE_SIZE
-#define PAGE_SIZE ( sysconf(_SC_PAGESIZE) / 1024 )
+#define PAGE_SIZE ( sysconf(_SC_PAGESIZE) )
 #endif
-
-#define PROCESS_COMM_LEN 300
+#define PAGE_SIZE_KB ( PAGE_SIZE / ONE_K )
 
 /*{
 
+#ifndef Process_isKernelThread
+#define Process_isKernelThread(_process) (_process->pgrp == 0)
+#endif
+
+#ifndef Process_isUserlandThread
+#define Process_isUserlandThread(_process) (_process->pid != _process->tgid)
+#endif
+
+#ifndef Process_isThread
+#define Process_isThread(_process) (Process_isUserlandThread(_process) || Process_isKernelThread(_process))
+#endif
+
 typedef enum ProcessField_ {
    PID = 1, COMM, STATE, PPID, PGRP, SESSION, TTY_NR, TPGID, FLAGS, MINFLT, CMINFLT, MAJFLT, CMAJFLT, UTIME,
    STIME, CUTIME, CSTIME, PRIORITY, NICE, ITREALVALUE, STARTTIME, VSIZE, RSS, RLIM, STARTCODE, ENDCODE,
@@ -47,7 +61,7 @@ typedef enum ProcessField_ {
    PROCESSOR, M_SIZE, M_RESIDENT, M_SHARE, M_TRS, M_DRS, M_LRS, M_DT, ST_UID, PERCENT_CPU, PERCENT_MEM,
    USER, TIME, NLWP, TGID,
    #ifdef HAVE_OPENVZ
-   VEID, VPID,
+   CTID, VPID,
    #endif
    #ifdef HAVE_VSERVER
    VXID,
@@ -55,6 +69,9 @@ typedef enum ProcessField_ {
    #ifdef HAVE_TASKSTATS
    RCHAR, WCHAR, SYSCR, SYSCW, RBYTES, WBYTES, CNCLWB, IO_READ_RATE, IO_WRITE_RATE, IO_RATE,
    #endif
+   #ifdef HAVE_CGROUP
+   CGROUP,
+   #endif
    LAST_PROCESSFIELD
 } ProcessField;
 
@@ -66,16 +83,18 @@ typedef struct Process_ {
    struct ProcessList_ *pl;
    bool updated;
 
-   unsigned int pid;
+   pid_t pid;
    char* comm;
    int indent;
    char state;
    bool tag;
-   unsigned int ppid;
+   bool showChildren;
+   bool show;
+   pid_t ppid;
    unsigned int pgrp;
    unsigned int session;
    unsigned int tty_nr;
-   unsigned int tgid;
+   pid_t tgid;
    int tpgid;
    unsigned long int flags;
    #ifdef DEBUG
@@ -91,9 +110,10 @@ typedef struct Process_ {
    long int priority;
    long int nice;
    long int nlwp;
+   char starttime_show[8];
+   time_t starttime_ctime;
    #ifdef DEBUG
    long int itrealvalue;
-   unsigned long int starttime;
    unsigned long int vsize;
    long int rss;
    unsigned long int rlim;
@@ -124,7 +144,7 @@ typedef struct Process_ {
    float percent_mem;
    char* user;
    #ifdef HAVE_OPENVZ
-   unsigned int veid;
+   unsigned int ctid;
    unsigned int vpid;
    #endif
    #ifdef HAVE_VSERVER
@@ -143,6 +163,9 @@ typedef struct Process_ {
    double io_rate_write_bps;
    unsigned long long io_rate_write_time;   
    #endif
+   #ifdef HAVE_CGROUP
+   char* cgroup;
+   #endif
 } Process;
 
 }*/
@@ -153,7 +176,7 @@ char* PROCESS_CLASS = "Process";
 #define PROCESS_CLASS NULL
 #endif
 
-char *Process_fieldNames[] = {
+const char *Process_fieldNames[] = {
    "", "PID", "Command", "STATE", "PPID", "PGRP", "SESSION",
    "TTY_NR", "TPGID", "FLAGS", "MINFLT", "CMINFLT", "MAJFLT", "CMAJFLT",
    "UTIME", "STIME", "CUTIME", "CSTIME", "PRIORITY", "NICE", "ITREALVALUE",
@@ -163,7 +186,7 @@ char *Process_fieldNames[] = {
    "M_TRS", "M_DRS", "M_LRS", "M_DT", "ST_UID", "PERCENT_CPU", "PERCENT_MEM",
    "USER", "TIME", "NLWP", "TGID", 
 #ifdef HAVE_OPENVZ
-   "VEID", "VPID",
+   "CTID", "VPID",
 #endif
 #ifdef HAVE_VSERVER
    "VXID",
@@ -172,28 +195,35 @@ char *Process_fieldNames[] = {
    "RCHAR", "WCHAR", "SYSCR", "SYSCW", "RBYTES", "WBYTES", "CNCLWB",
    "IO_READ_RATE", "IO_WRITE_RATE", "IO_RATE",
 #endif
+#ifdef HAVE_CGROUP
+   "CGROUP",
+#endif
 "*** report bug! ***"
 };
 
-char *Process_fieldTitles[] = {
+const char *Process_fieldTitles[] = {
    "", "  PID ", "Command ", "S ", " PPID ", " PGRP ", " SESN ",
    "  TTY ", "TPGID ", "- ", "- ", "- ", "- ", "- ",
    " UTIME+  ", " STIME+  ",  "- ", "- ", "PRI ", " NI ", "- ",
-   "- ", "- ", "- ", "- ", "- ", "- ", "- ",
+   "START ", "- ", "- ", "- ", "- ", "- ", "- ",
    "- ", "- ", "- ", "- ", "- ", "- ", "- ",
    "- ", "- ", "- ", "CPU ", " VIRT ", "  RES ", "  SHR ",
    " CODE ", " DATA ", " LIB ", " DIRTY ", " UID ", "CPU% ", "MEM% ",
    "USER     ", "  TIME+  ", "NLWP ", " TGID ",
 #ifdef HAVE_OPENVZ
-   " VEID ", " VPID ",
+   " CTID ", " VPID ",
 #endif
 #ifdef HAVE_VSERVER
    " VXID ",
 #endif
 #ifdef HAVE_TASKSTATS
-   "   RD_CHAR ", "   WR_CHAR ", "   RD_SYSC ", "   WR_SYSC ", "     IO_RD ", "     IO_WR ", " IO_CANCEL ",
+   "   RD_CHAR ", "   WR_CHAR ", "   RD_SYSC ", "   WR_SYSC ", " IO_RBYTES ", " IO_WBYTES ", " IO_CANCEL ",
    " IORR ", " IOWR ", "   IO ",
 #endif
+#ifdef HAVE_CGROUP
+   "    CGROUP ",
+#endif
+"*** report bug! ***"
 };
 
 static int Process_getuid = -1;
@@ -202,13 +232,18 @@ static int Process_getuid = -1;
 #define ONE_M (ONE_K * ONE_K)
 #define ONE_G (ONE_M * ONE_K)
 
-static void Process_printLargeNumber(Process* this, RichString *str, unsigned long number) {
+static void Process_printLargeNumber(Process* this, RichString* str, unsigned long number) {
    char buffer[11];
    int len;
-   if(number >= (1000 * ONE_M)) {
-      len = snprintf(buffer, 10, "%4.2fG ", (float)number / ONE_M);
-      RichString_appendn(str, CRT_colors[LARGE_NUMBER], buffer, len);
-   } else if(number >= (100000)) {
+   if(number >= (10 * ONE_M)) {
+      if(number >= (100 * ONE_M)) {
+         len = snprintf(buffer, 10, "%4ldG ", number / ONE_M);
+         RichString_appendn(str, CRT_colors[LARGE_NUMBER], buffer, len);
+      } else {
+         len = snprintf(buffer, 10, "%3.1fG ", (float)number / ONE_M);
+         RichString_appendn(str, CRT_colors[LARGE_NUMBER], buffer, len);
+      }
+   } else if (number >= 100000) {
       len = snprintf(buffer, 10, "%4ldM ", number / ONE_K);
       int attr = this->pl->highlightMegabytes
                ? CRT_colors[PROCESS_MEGABYTES]
@@ -251,10 +286,10 @@ static void Process_printTime(RichString* str, unsigned long t) {
 }
 
 static inline void Process_writeCommand(Process* this, int attr, int baseattr, RichString* str) {
-   int start = str->len;
+   int start = RichString_size(str);
    RichString_append(str, attr, this->comm);
    if (this->pl->highlightBaseName) {
-      int finish = str->len - 1;
+      int finish = RichString_size(str) - 1;
       int space = RichString_findChar(str, ' ', start);
       if (space != -1)
          finish = space - 1;
@@ -284,10 +319,10 @@ static inline void Process_outputRate(Process* this, RichString* str, int attr,
 }
 
 static void Process_writeField(Process* this, RichString* str, ProcessField field) {
-   char buffer[PROCESS_COMM_LEN];
+   char buffer[128]; buffer[127] = '\0';
    int attr = CRT_colors[DEFAULT_COLOR];
    int baseattr = CRT_colors[PROCESS_BASENAME];
-   int n = PROCESS_COMM_LEN;
+   int n = sizeof(buffer) - 1;
 
    switch (field) {
    case PID: snprintf(buffer, n, "%5u ", this->pid); break;
@@ -300,7 +335,7 @@ static void Process_writeField(Process* this, RichString* str, ProcessField fiel
    case PROCESSOR: snprintf(buffer, n, "%3d ", this->processor+1); break;
    case NLWP: snprintf(buffer, n, "%4ld ", this->nlwp); break;
    case COMM: {
-      if (this->pl->highlightThreads && (this->pid != this->tgid || this->m_size == 0)) {
+      if (this->pl->highlightThreads && Process_isThread(this)) {
          attr = CRT_colors[PROCESS_THREAD];
          baseattr = CRT_colors[PROCESS_THREAD_BASENAME];
       }
@@ -322,9 +357,9 @@ static void Process_writeField(Process* this, RichString* str, ProcessField fiel
             n -= 4;
          }
          if (this->pl->direction == 1)
-            snprintf(buf, n, " `- ");
+            snprintf(buf, n, " `%s ", this->showChildren ? "-" : "+" );
          else
-            snprintf(buf, n, " ,- ");
+            snprintf(buf, n, " ,%s ", this->showChildren ? "-" : "+" );
          RichString_append(str, CRT_colors[PROCESS_TREE], buffer);
          Process_writeCommand(this, attr, baseattr, str);
          return;
@@ -351,16 +386,16 @@ static void Process_writeField(Process* this, RichString* str, ProcessField fiel
            : attr;
       break;
    }
-   case M_DRS: Process_printLargeNumber(this, str, this->m_drs * PAGE_SIZE); return;
-   case M_DT: Process_printLargeNumber(this, str, this->m_dt * PAGE_SIZE); return;
-   case M_LRS: Process_printLargeNumber(this, str, this->m_lrs * PAGE_SIZE); return;
-   case M_TRS: Process_printLargeNumber(this, str, this->m_trs * PAGE_SIZE); return;
-   case M_SIZE: Process_printLargeNumber(this, str, this->m_size * PAGE_SIZE); return;
-   case M_RESIDENT: Process_printLargeNumber(this, str, this->m_resident * PAGE_SIZE); return;
-   case M_SHARE: Process_printLargeNumber(this, str, this->m_share * PAGE_SIZE); return;
+   case M_DRS: Process_printLargeNumber(this, str, this->m_drs * PAGE_SIZE_KB); return;
+   case M_DT: Process_printLargeNumber(this, str, this->m_dt * PAGE_SIZE_KB); return;
+   case M_LRS: Process_printLargeNumber(this, str, this->m_lrs * PAGE_SIZE_KB); return;
+   case M_TRS: Process_printLargeNumber(this, str, this->m_trs * PAGE_SIZE_KB); return;
+   case M_SIZE: Process_printLargeNumber(this, str, this->m_size * PAGE_SIZE_KB); return;
+   case M_RESIDENT: Process_printLargeNumber(this, str, this->m_resident * PAGE_SIZE_KB); return;
+   case M_SHARE: Process_printLargeNumber(this, str, this->m_share * PAGE_SIZE_KB); return;
    case ST_UID: snprintf(buffer, n, "%4d ", this->st_uid); break;
    case USER: {
-      if (Process_getuid != this->st_uid)
+      if (Process_getuid != (int) this->st_uid)
          attr = CRT_colors[PROCESS_SHADOW];
       if (this->user) {
       snprintf(buffer, n, "%-8s ", this->user);
@@ -396,8 +431,9 @@ static void Process_writeField(Process* this, RichString* str, ProcessField fiel
       }
       break;
    }
+   case STARTTIME: snprintf(buffer, n, "%s", this->starttime_show); break;
    #ifdef HAVE_OPENVZ
-   case VEID: snprintf(buffer, n, "%5u ", this->veid); break;
+   case CTID: snprintf(buffer, n, "%5u ", this->ctid); break;
    case VPID: snprintf(buffer, n, "%5u ", this->vpid); break;
    #endif
    #ifdef HAVE_VSERVER
@@ -415,6 +451,9 @@ static void Process_writeField(Process* this, RichString* str, ProcessField fiel
    case IO_WRITE_RATE: Process_outputRate(this, str, attr, buffer, n, this->io_rate_write_bps); return;
    case IO_RATE: Process_outputRate(this, str, attr, buffer, n, this->io_rate_read_bps + this->io_rate_write_bps); return;
    #endif
+   #ifdef HAVE_CGROUP
+   case CGROUP: snprintf(buffer, n, "%-10s ", this->cgroup); break;
+   #endif
 
    default:
       snprintf(buffer, n, "- ");
@@ -425,10 +464,10 @@ static void Process_writeField(Process* this, RichString* str, ProcessField fiel
 static void Process_display(Object* cast, RichString* out) {
    Process* this = (Process*) cast;
    ProcessField* fields = this->pl->fields;
-   RichString_init(out);
+   RichString_prune(out);
    for (int i = 0; fields[i]; i++)
       Process_writeField(this, out, fields[i]);
-   if (this->pl->shadowOtherUsers && this->st_uid != Process_getuid)
+   if (this->pl->shadowOtherUsers && (int)this->st_uid != Process_getuid)
       RichString_setAttr(out, CRT_colors[PROCESS_SHADOW]);
    if (this->tag == true)
       RichString_setAttr(out, CRT_colors[PROCESS_TAG]);
@@ -439,6 +478,9 @@ void Process_delete(Object* cast) {
    Process* this = (Process*) cast;
    assert (this != NULL);
    if (this->comm) free(this->comm);
+#ifdef HAVE_CGROUP
+   if (this->cgroup) free(this->cgroup);
+#endif
    free(this);
 }
 
@@ -450,36 +492,20 @@ Process* Process_new(struct ProcessList_ *pl) {
    this->pid = 0;
    this->pl = pl;
    this->tag = false;
+   this->showChildren = true;
+   this->show = true;
    this->updated = false;
    this->utime = 0;
    this->stime = 0;
    this->comm = NULL;
    this->indent = 0;
+#ifdef HAVE_CGROUP
+   this->cgroup = NULL;
+#endif
    if (Process_getuid == -1) Process_getuid = getuid();
    return this;
 }
 
-Process* Process_clone(Process* this) {
-   Process* clone = malloc(sizeof(Process));
-   #if HAVE_TASKSTATS
-   this->io_rchar = 0;
-   this->io_wchar = 0;
-   this->io_syscr = 0;
-   this->io_syscw = 0;
-   this->io_read_bytes = 0;
-   this->io_rate_read_bps = 0;
-   this->io_rate_read_time = 0;
-   this->io_write_bytes = 0;
-   this->io_rate_write_bps = 0;
-   this->io_rate_write_time = 0;
-   this->io_cancelled_write_bytes = 0;
-   #endif
-   memcpy(clone, this, sizeof(Process));
-   this->comm = NULL;
-   this->pid = 0;
-   return clone;
-}
-
 void Process_toggleTag(Process* this) {
    this->tag = this->tag == true ? false : true;
 }
@@ -493,6 +519,7 @@ bool Process_setPriority(Process* this, int priority) {
    return (err == 0);
 }
 
+#ifdef HAVE_PLPA
 unsigned long Process_getAffinity(Process* this) {
    unsigned long mask = 0;
    plpa_sched_getaffinity(this->pid, sizeof(unsigned long), (plpa_cpu_set_t*) &mask);
@@ -502,9 +529,10 @@ unsigned long Process_getAffinity(Process* this) {
 bool Process_setAffinity(Process* this, unsigned long mask) {
    return (plpa_sched_setaffinity(this->pid, sizeof(unsigned long), (plpa_cpu_set_t*) &mask) == 0);
 }
+#endif
 
-void Process_sendSignal(Process* this, int signal) {
-   kill(this->pid, signal);
+void Process_sendSignal(Process* this, int sgn) {
+   kill(this->pid, sgn);
 }
 
 int Process_pidCompare(const void* v1, const void* v2) {
@@ -530,9 +558,13 @@ int Process_compare(const void* v1, const void* v2) {
    case PPID:
       return (p1->ppid - p2->ppid);
    case USER:
-      return strcmp(p1->user, p2->user);
+      return strcmp(p1->user ? p1->user : "", p2->user ? p2->user : "");
    case PRIORITY:
       return (p1->priority - p2->priority);
+   case PROCESSOR:
+      return (p1->processor - p2->processor);
+   case SESSION:
+      return (p1->session - p2->session);
    case STATE:
       return (p1->state - p2->state);
    case NICE:
@@ -565,9 +597,15 @@ int Process_compare(const void* v1, const void* v2) {
       return strcmp(p1->comm, p2->comm);
    case NLWP:
       return (p1->nlwp - p2->nlwp);
+   case STARTTIME: {
+      if (p1->starttime_ctime == p2->starttime_ctime)
+         return (p1->pid - p2->pid);
+      else
+         return (p1->starttime_ctime - p2->starttime_ctime);
+   }
    #ifdef HAVE_OPENVZ
-   case VEID:
-      return (p1->veid - p2->veid);
+   case CTID:
+      return (p1->ctid - p2->ctid);
    case VPID:
       return (p1->vpid - p2->vpid);
    #endif
@@ -587,6 +625,10 @@ int Process_compare(const void* v1, const void* v2) {
    case IO_WRITE_RATE: diff = p2->io_rate_write_bps - p1->io_rate_write_bps; goto test_diff;
    case IO_RATE: diff = (p2->io_rate_read_bps + p2->io_rate_write_bps) - (p1->io_rate_read_bps + p1->io_rate_write_bps); goto test_diff;
    #endif
+   #ifdef HAVE_CGROUP
+   case CGROUP:
+      return strcmp(p1->cgroup ? p1->cgroup : "", p2->cgroup ? p2->cgroup : "");
+   #endif
 
    default:
       return (p1->pid - p2->pid);
