@@ -1,6 +1,6 @@
 /*
 htop - UsersTable.c
-(C) 2004-2006 Hisham H. Muhammad
+(C) 2004-2010 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
@@ -35,7 +35,7 @@ void UsersTable_delete(UsersTable* this) {
    free(this);
 }
 
-char* UsersTable_getRef(UsersTable* this, int uid) {
+char* UsersTable_getRef(UsersTable* this, unsigned int uid) {
    char* name = (char*) (Hashtable_get(this->users, uid));
    if (name == NULL) {
       struct passwd* userData = getpwuid(uid);
@@ -47,10 +47,6 @@ char* UsersTable_getRef(UsersTable* this, int uid) {
    return name;
 }
 
-inline int UsersTable_size(UsersTable* this) {
-   return (Hashtable_size(this->users));
-}
-
 inline void UsersTable_foreach(UsersTable* this, Hashtable_PairFunction f, void* userData) {
    Hashtable_foreach(this->users, f, userData);
 }
